/************************************************************************/
/*  Command line parsing routines.					*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Berkeley Seismological Laboratory				*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2014 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static const char sccsid[] = "$Id: parse_cmdline.c,v 1.2 2014/10/07 00:06:24 doug Exp $ ";
#endif

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "qlib2.h"

#include "externals.h"

#define	argc	(*pargc)
#define	argv	(*pargv)

double time_seconds(char *str);

/************************************************************************/
/*  parse_cmdline:							*/
/*	Parse command line.						*/
/************************************************************************/
int parse_cmdline 
   (int		*pargc,		/* ptr to argc.				*/
    char	***pargv)	/* ptr to argv.				*/
{
    /* Variables needed for getopt. */
    extern char	*optarg;
    extern int	optind, opterr;
    int		c;

    stream_tol = -1;
    cmdname = trim(*argv);
    /*	Parse command line options.					*/
    while ( (c = getopt(argc,argv,"hnars:q:o:G:d:D:")) != -1)
	switch (c) {
	case '?':
	case 'h':   print_syntax(cmdname,syntax,info); exit(0); break;
	case 'n':   ++no_output; break;
	case 'a':   ++append_output; break;
	case 'r':   ++reversed; break;
	case 's':   keep_usecs = time_seconds(optarg) * USECS_PER_SEC; break;
	case 'q':   min_clock_quality = atoi(optarg); break;
	case 'o':   output_file = optarg; break;
	case 'G':   stream_tol = atoi(optarg) * USECS_PER_TICK; break;
	case 'd':   debug_option = atoi(optarg); break;
	case 'D':
	    /* Date display format.   */
	    if (strcmp (optarg, "j") == 0) date_fmt = JULIAN_FMT;
	    else if (strcmp (optarg, "j,") == 0) date_fmt = JULIAN_FMT_1;
	    else if (strcmp (optarg, "m") == 0) date_fmt = MONTH_FMT;
	    else if (strcmp (optarg, "m,") == 0) date_fmt = MONTH_FMT_1;
	    else {
		fprintf (info, "Error: invalid format for date display: %s\n", optarg);
		exit(1);
	    }
	    break;
	}

    if (output_file == NULL) info = stderr;

    outmode = (append_output) ? "a" : "w";
    
    /*	Skip over all options and their arguments.			*/
    argv = &(argv[optind]);
    argc -= optind;

    if (argc > 0) input_file = *argv;

    /* The remaining arguments are the input files.  If there are none,	*/
    /* then assume 1 file from stdin.					*/
    return(0);
}

/************************************************************************/
/*  time_seconds:							*/
/*	Compute number of seconds for the specified time.		*/
/************************************************************************/
double time_seconds 
   (char	*span)		/* timespan string.			*/
{
    char	*p;
    int		l;
    int		span_value;
    double	fseconds;

    /*	Compute time in seconds based on specified span value.		*/
    /*	Units for span values can be:					*/
    /*	    S (or nothing) -	Seconds.				*/
    /*	    F -			Fractional seconds			*/
    /*	    y -			Year.					*/
    /*	    M -			Minutes.				*/
    /*	    H -			Hours.					*/

    span_value = strtol(span,&p,10);
    if ( p == span || span_value < 0 || (l=strlen(p) > 1) ) {
	fprintf (stderr, "invalid span value: %s\n", span);
	exit(1);
    }

    switch (*p) {
      case  0:
      case 'S':	fseconds = (double)span_value; break;
      case 'F':	fseconds = (double)span_value / TICKS_PER_SEC; break;
      case 'U':	fseconds = (double)span_value / USECS_PER_SEC; break;
      case 'M':	fseconds = (double)span_value * SEC_PER_MINUTE; break;
      case 'H':	fseconds = (double)span_value * SEC_PER_HOUR; break;
      default:
	fprintf (stderr, "invalid span value: %s\n", span);
	exit(1);
    }
    return (fseconds);
}

